r"""Compute spectral distortion for DM annihilation scenarios (s and p-wave)"""

import Greens as G
import numpy as np
import wrap_emcee
import h5py
import matplotlib.pyplot as mat

nthreadloc=4
usewalkers=200
nsamples  =5000000
addnoisetodata=0


#=======================================================================================
# scenario small
#=======================================================================================
outname="DM_annihilation_s_p_wave.2.0e-23.1.0e-28.fac_40.hd5"
fannlabel="f_{\\rm ann, s}\\,[ 10^{-23} {\\rm eV \\, sec^{-1} } ]"
fannplabel="f_{\\rm ann,p}\\,[ 10^{-28} {\\rm eV \\, sec^{-1} } ]"

PIXIE_fac=40.0

fann_model=2.0e-23
rescalev3=1.0e-23

fannp_model=1.0e-28
rescalev4=1.0e-28

#=======================================================================================
# scenario large
#=======================================================================================
outname="DM_annihilation_s_p_wave.1.0e-22.1.0e-26.fac_20.hd5"
fannlabel="f_{\\rm ann, s}\\,[ 10^{-22} {\\rm eV \\, sec^{-1} } ]"
fannplabel="f_{\\rm ann,p}\\,[ 10^{-26} {\\rm eV \\, sec^{-1} } ]"

PIXIE_fac=20.0

fann_model=1.0e-22
rescalev3=1.0e-22

fannp_model=1.0e-26
rescalev4=1.0e-26

#=======================================================================================
# common variables
#=======================================================================================
Gamma=0.0
f_X=0.0

DeltaT=1.2e-4
rescalev1=1.0e-9

yre_model=4.0e-7
rescalev2=1.0e-7

priorv1 =np.array([-DeltaT*10.0,  DeltaT*10.0])/rescalev1
priorv2 =np.array([0.0,  yre_model*100.0])     /rescalev2
priorv3 =np.array([0.0, fann_model*100.0])     /rescalev3
priorv4 =np.array([0.0, fannp_model*100.0])    /rescalev4


#=======================================================================================
def SD_model(bands=None, v1=None, v2=None, v3=None, v4=None):
    r"""Compute spectral distortion for DM annihilation + reionization scenario"""

    #===================================================================================
    # check prior
    #===================================================================================
    nf=len(bands)
    bands=np.ones(nf)*1.0e+10
    
    if(v1 < priorv1[0]) or (v1 > priorv1[1]) or \
      (v2 < priorv2[0]) or (v2 > priorv2[1]) or \
      (v3 < priorv3[0]) or (v3 > priorv3[1]) or \
      (v4 < priorv4[0]) or (v4 > priorv4[1]):
        return bands
    
    #===================================================================================
    Del  =v1*rescalev1
    y_re =v2*rescalev2
    f_ann=v3*rescalev3
    f_annp=v4*rescalev4

    #print "sample at: Delta = ", Del, " y_re = ", y_re, " f_ann = ", f_ann, " f_annp = ", f_annp

    G.compute_mock_data_binned(bands,
                               f_ann, 0,
                               0, 0, 0, 0,
                               Gamma, f_X,
                               y_re, Del+DeltaT, 0)

    bandsp=np.ones(nf)*1.0e+10

    G.compute_mock_data_binned(bandsp,
                               f_annp, 1,
                               0, 0, 0, 0,
                               0, 0, 0, 0, 0)

    bands+=bandsp

    return bands


#=======================================================================================
def DM_annihilation_estimation(filename):
    r"""set up the parameters and run the chain"""

    #===================================================================================
    # frequencies
    #===================================================================================
    nu_GHz_min=30.0
    nu_GHz_max=1000.0
    Dnu_GHz=15.0
    
    #===================================================================================
    # initialize Greens function
    #===================================================================================
    nfreq=G.initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz)

    #===================================================================================
    # central frequencies
    #===================================================================================
    nu_GHzarr=np.array([nu_GHz_min+i*Dnu_GHz for i in range(nfreq)])
    
    #===================================================================================
    # mock data with noise
    #===================================================================================
    Sigarr=np.ones(nfreq)
    Sigarrp=np.ones(nfreq)
    Garr=np.ones(nfreq)
    YSZarr=np.ones(nfreq)

    Del=DeltaT
    fann=fann_model
    fannp=fannp_model
    ypar=yre_model

    DI=5.0e-26 / PIXIE_fac
    
    G.compute_mock_data_binned(Sigarr, fann, 0, 0, 0, 0, 0, Gamma, f_X, ypar, Del, addnoisetodata*DI)
    G.compute_mock_data_binned(Garr, 0, 0, 0, 0, 0, 0, 0, 0, 0, Del, 0)
    G.compute_mock_data_binned(YSZarr, 0, 0, 0, 0, 0, 0, 0, 0, ypar, 0, 0)
    
    # get p-wave signal
    G.compute_mock_data_binned(Sigarrp, fannp, 1, 0, 0, 0, 0, 0, 0, 0, 0, addnoisetodata*DI)
    
    Sigarr+=Sigarrp
    
    G.set_verbosity(-1)
    
    #===================================================================================
    # errors and covariance
    #===================================================================================
    errarr=np.ones(nfreq)*DI*1.0e+26
    meas_cov = np.diag(errarr * errarr)

    #mat.plot(nu_GHzarr, Sigarr, 'r-')
    #mat.plot(nu_GHzarr, YSZarr,'b--')
    #mat.plot(nu_GHzarr, Garr,'r--')
    #mat.show()
    #print meas_cov
    
    #===================================================================================
    # MCMC parameters
    #===================================================================================
    default_params = {}
    default_params['bands'] = nu_GHzarr
    default_params['v1'] = 0.0            /rescalev1
    default_params['v2'] = yre_model      /rescalev2
    default_params['v3'] = fann_model     /rescalev3
    default_params['v4'] = fannp_model    /rescalev4

    Del_fit = {"kwarg_name": "v1",
               "kwarg_desc": "\\Delta^\\ast\\,[ 10^{-9} ]",
               "walker_range": priorv1,
               "prior_mean": 0.0,
               "prior_std": 0.0005/rescalev1
               }
    
    y_re_fit = {"kwarg_name": "v2",
                "kwarg_desc": "y_{\\rm re}\\,[ 10^{-7} ]",
                "walker_range": priorv2
               }

    f_ann_fit = {"kwarg_name": "v3",
                 "kwarg_desc": fannlabel,
                 "walker_range": priorv3
                }

    f_annp_fit = {"kwarg_name": "v4",
                  "kwarg_desc": fannplabel,
                  "walker_range": priorv4
                  }

    #===================================================================================
    # choose fit
    #===================================================================================
    fit_list = [Del_fit, y_re_fit, f_ann_fit, f_annp_fit]

    input_values = {}
    input_values['v1'] = 0
    input_values['v2'] = yre_model /rescalev2
    input_values['v3'] = fann_model/rescalev3
    input_values['v4'] = fannp_model/rescalev4

    wrap_emcee.call_mcmc(Sigarr, meas_cov, default_params, input_values,
                         fit_list, "DM_annihilation_spwave_fit.SD_model",
                         nwalkers=usewalkers, n_burn=200, n_run=nsamples/usewalkers,
                         threads=nthreadloc, verbose=False, outfile=filename)



#=======================================================================================
#
# main call of code
#
#=======================================================================================
np.seterr(all='ignore')
if __name__ == '__main__':
    DM_annihilation_estimation(outname)


#=======================================================================================
#=======================================================================================

