//==================================================================================================
//
// This code can be used to compute an approximation for the spectral distortion caused by different
// energy releasee mechanisms. The computation is fast and based on the Greens function computed for
// a fixed background cosmology.
//
//==================================================================================================
//
// Author: Jens Chluba
// first implementation: April  2013
// last modification   : August 2013
//
//==================================================================================================
// 13.10.2013: changed initialization sequence for Greens function
// 30.08.2013: added runmodes and setup for step and bend power spectrum cases

//==================================================================================================
// Standards
//==================================================================================================
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <ctime>
#include <iomanip>
#include <cmath>
#include <limits.h>
#include <vector>

//==================================================================================================
// required libs
//==================================================================================================
#include "Greens.h"

//==================================================================================================
// namespaces
//==================================================================================================
using namespace std;

//==================================================================================================
//
// main call of CosmoTherm. Here parameters are handled.
//
//==================================================================================================
int main(int narg, char *args[])
{
    string filename, path, ending, mode="default";
    
    if(narg==2) filename=args[narg-1];
    else if(narg==3){ mode=args[narg-2]; filename=args[narg-1]; }
    else{ cerr << " Too many/few parameters " << endl; exit(1); }

    //==============================================================================================
    //
    // read startup
    //
    //==============================================================================================
    ifstream params(filename.c_str());
    
    double Delta;
    double f_ann; int wave;
    double A, k0, nS, nrun;
    double Astep, kstep, nSstep;
    double kbend, nSbend;
    double Gamma_X, f_X;
    double APP, ki;
    double y;

    params >> Delta;

    params >> f_ann;
    params >> wave;
    //
    params >> A; params >> k0; params >> nS; params >> nrun;
    //
    params >> Astep; params >> kstep; params >> nSstep;
    //
    params >> kbend; params >> nSbend;
    //
    params >> Gamma_X; params >> f_X;
    //
    params >> APP; params >> ki;
    //
    params >> y;

    params >> path;
    params >> ending;
    
    //==============================================================================================
    //
    // check some of the parameters
    //
    //==============================================================================================
    if(kbend<=0) kbend=1.0e+30;
    if(Gamma_X<=0){ f_X=0.0; Gamma_X=1.0; }
    
    //==============================================================================================
    //
    // Settings for simple mock data
    //
    //==============================================================================================
    double noise=0.0e-26;
    double nu_GHz_min=30.0;
    double nu_GHz_max=1000.0;
    double Dnu_GHz=15.0;    

    //==============================================================================================
    //
    // initialize Greens-function
    //
    //==============================================================================================
    if(mode=="default" || mode=="Greens" || mode=="Mock") initialize_Greens_function();
    if(mode=="Mockpre" || mode=="PCA") initialize_Greens_function(nu_GHz_min, nu_GHz_max, Dnu_GHz);
    
    Set_P_step_params(Astep, kstep, nSstep);
    Set_P_bend_params(kbend, nSbend);
    Set_P_PPro_params(APP, ki);

    //==============================================================================================
    //
    // perform calculation of spectrum according to setting
    //
    //==============================================================================================
    if(mode=="default")
    {
        output_distortion(path+"distortion"+ending,
                          f_ann, wave,
                          A, k0, nS, nrun,
                          Gamma_X, f_X, y, Delta);
        
        output_energy_release_history(path+"heating_rate"+ending,
                                      f_ann, wave,
                                      A, k0, nS, nrun,
                                      Gamma_X, f_X);
    }

    //==============================================================================================
    //
    // output of Greens function at different redshifts; this is just for playing with the method
    //
    //==============================================================================================
    else if(mode=="Greens")
    {
        output_Greens_function(path+"Green"+ending, 1.000001e+3, 4.9999e+6, 40);
        output_y_visibility_function(path+"J_y"+ending, 1.000001e+3, 3.0e+5, 50);
        output_best_fit_mu_y_Decay(path+"Decay_mu_y"+ending, 1.0, 5.0e+3, 5.0e+5, 100);
    }
    
    //==============================================================================================
    //
    // output mock data for distortion. The integrals over frequency are directly carried out.
    //
    //==============================================================================================
    else if(mode=="Mock")
    {
        compute_mock_data(path+"distortion_mock"+ending,
                          f_ann, wave,
                          A, k0, nS, nrun,
                          Gamma_X, f_X, y, Delta,
                          nu_GHz_min, nu_GHz_max, Dnu_GHz, noise);
    }
    
    //==============================================================================================
    //
    // output mock data for distortion. This should give the same result as mode=="Mock", but here
    // the Greens function was initially binned.
    //
    //==============================================================================================
    else if(mode=="Mockpre")
    {
        compute_mock_data_binned(path+"distortion_mock_binned"+ending,
                                 f_ann, wave,
                                 A, k0, nS, nrun,
                                 Gamma_X, f_X, y, Delta, noise);
    }

    //==============================================================================================
    //
    // outputs for PCA method. This is not of common interest at this point
    //
    //==============================================================================================
    else if(mode=="PCA")
    {
        output_Greens_function_matrix(path+"Greens_matrix.caseI.mid.M.nu_min_30.Dnu_15"+ending, 1.0000001e+3, 4.999999e+6, 400);
        //output_Greens_function_matrix(path+"Greens_matrix.caseIII.nu_min_30.Dnu_15"+ending, 3.0e+3, 5.0e+5, 400);
        //output_Greens_function_matrix(path+"Greens_matrix.res.nar.nu_min_30.Dnu_15"+ending, 8.0e+3, 4.0e+5, 400);
    }
    
    
    return 0;
}

//==================================================================================================
//==================================================================================================
